@@ -13,7 +13,7 @@ use crate::{
     types::{
         self, CreateNamespaceVars, DeployApp, DeployAppConnection, DeployAppVersion,
         DeployAppVersionConnection, DnsDomain, GetAppTemplateFromSlugVariables,
-        GetAppTemplatesQueryVariables, GetCurrentUserWithAppsVars, GetDeployAppAndVersion,
+        GetAppTemplatesVars, GetCurrentUserWithAppsVars, GetDeployAppAndVersion,
         GetDeployAppVersionsVars, GetNamespaceAppsVars, GetSignedUrlForPackageUploadVariables, Log,
         LogStream, PackageVersionConnection, PublishDeployAppVars, PushPackageReleasePayload,
         SignedUrl, TagPackageReleasePayload, UpsertDomainFromZoneFileVars,
@@ -74,18 +74,77 @@ pub async fn fetch_app_templates(
     client: &WasmerClient,
     category_slug: String,
     first: i32,
+    after: Option<String>,
+    sort_by: Option<types::AppTemplatesSortBy>,
 ) -> Result<Option<types::AppTemplateConnection>, anyhow::Error> {
     client
-        .run_graphql_strict(types::GetAppTemplatesQuery::build(
-            GetAppTemplatesQueryVariables {
-                category_slug,
-                first,
-            },
-        ))
+        .run_graphql_strict(types::GetAppTemplates::build(GetAppTemplatesVars {
+            category_slug,
+            first,
+            after,
+            sort_by,
+        }))
         .await
         .map(|r| r.get_app_templates)
 }
 
+/// Fetch all app templates by paginating through the responses.
+///
+/// Will fetch at most `max` templates.
+pub fn fetch_all_app_templates(
+    client: &WasmerClient,
+    page_size: i32,
+    sort_by: Option<types::AppTemplatesSortBy>,
+) -> impl futures::Stream<Item = Result<Vec<types::AppTemplate>, anyhow::Error>> + '_ {
+    let vars = GetAppTemplatesVars {
+        category_slug: String::new(),
+        first: page_size,
+        sort_by,
+        after: None,
+    };
+
+    futures::stream::try_unfold(
+        Some(vars),
+        move |vars: Option<types::GetAppTemplatesVars>| async move {
+            let vars = match vars {
+                Some(vars) => vars,
+                None => return Ok(None),
+            };
+
+            let con = client
+                .run_graphql_strict(types::GetAppTemplates::build(vars.clone()))
+                .await?
+                .get_app_templates
+                .context("backend did not return any data")?;
+
+            let items = con
+                .edges
+                .into_iter()
+                .flatten()
+                .filter_map(|edge| edge.node)
+                .collect::<Vec<_>>();
+
+            let next_cursor = con
+                .page_info
+                .end_cursor
+                .filter(|_| con.page_info.has_next_page);
+
+            let next_vars = next_cursor.map(|after| types::GetAppTemplatesVars {
+                after: Some(after),
+                ..vars
+            });
+
+            #[allow(clippy::type_complexity)]
+            let res: Result<
+                Option<(Vec<types::AppTemplate>, Option<types::GetAppTemplatesVars>)>,
+                anyhow::Error,
+            > = Ok(Some((items, next_vars)));
+
+            res
+        },
+    )
+}
+
 /// Get a signed URL to upload packages.
 pub async fn get_signed_url_for_package_upload(
     client: &WasmerClient,
@@ -95,7 +154,7 @@ pub async fn get_signed_url_for_package_upload(
     version: Option<&str>,
 ) -> Result<Option<SignedUrl>, anyhow::Error> {
     client
-        .run_graphql(types::GetSignedUrlForPackageUpload::build(
+        .run_graphql_strict(types::GetSignedUrlForPackageUpload::build(
             GetSignedUrlForPackageUploadVariables {
                 expires_after_seconds,
                 filename,
@@ -164,7 +223,15 @@ pub async fn tag_package_release(
         .map(|r| r.tag_package_release)
 }
 
-/// Get the currently logged in used, together with all accessible namespaces.
+/// Get the currently logged in user.
+pub async fn current_user(client: &WasmerClient) -> Result<Option<types::User>, anyhow::Error> {
+    client
+        .run_graphql(types::GetCurrentUser::build(()))
+        .await
+        .map(|x| x.viewer)
+}
+
+/// Get the currently logged in user, together with all accessible namespaces.
 ///
 /// You can optionally filter the namespaces by the user role.
 pub async fn current_user_with_namespaces(
@@ -172,9 +239,9 @@ pub async fn current_user_with_namespaces(
     namespace_role: Option<types::GrapheneRole>,
 ) -> Result<types::UserWithNamespaces, anyhow::Error> {
     client
-        .run_graphql(types::GetCurrentUser::build(types::GetCurrentUserVars {
-            namespace_role,
-        }))
+        .run_graphql(types::GetCurrentUserWithNamespaces::build(
+            types::GetCurrentUserWithNamespacesVars { namespace_role },
+        ))
         .await?
         .viewer
         .context("not logged in")
@@ -544,9 +611,11 @@ pub async fn user_accessible_apps(
 
     // Get all aps in user-accessible namespaces.
     let namespace_res = client
-        .run_graphql(types::GetCurrentUser::build(types::GetCurrentUserVars {
-            namespace_role: None,
-        }))
+        .run_graphql(types::GetCurrentUserWithNamespaces::build(
+            types::GetCurrentUserWithNamespacesVars {
+                namespace_role: None,
+            },
+        ))
         .await?;
     let active_user = namespace_res.viewer.context("not logged in")?;
     let namespace_names = active_user
@@ -655,9 +724,11 @@ pub async fn user_namespaces(
     client: &WasmerClient,
 ) -> Result<Vec<types::Namespace>, anyhow::Error> {
     let user = client
-        .run_graphql(types::GetCurrentUser::build(types::GetCurrentUserVars {
-            namespace_role: None,
-        }))
+        .run_graphql(types::GetCurrentUserWithNamespaces::build(
+            types::GetCurrentUserWithNamespacesVars {
+                namespace_role: None,
+            },
+        ))
         .await?
         .viewer
         .context("not logged in")?;
@@ -1137,6 +1208,20 @@ pub fn get_all_dns_records_stream(
     )
 }
 
+pub async fn purge_cache_for_app_version(
+    client: &WasmerClient,
+    vars: types::PurgeCacheForAppVersionVars,
+) -> Result<(), anyhow::Error> {
+    client
+        .run_graphql_strict(types::PurgeCacheForAppVersion::build(vars))
+        .await
+        .map_err(anyhow::Error::from)
+        .map(|x| x.purge_cache_for_app_version)
+        .context("backend did not return data")?;
+
+    Ok(())
+}
+
 /// Convert a [`OffsetDateTime`] to a unix timestamp that the WAPM backend
 /// understands.
 fn unix_timestamp(ts: OffsetDateTime) -> f64 {
