@@ -1,19 +1,28 @@
 //! Create a new Edge app.
 
+use std::{
+    collections::HashMap,
+    env,
+    io::Cursor,
+    path::{Path, PathBuf},
+    str::FromStr,
+    time::Duration,
+};
+
+use anyhow::Context;
+use colored::Colorize;
+use dialoguer::{theme::ColorfulTheme, Confirm, Select};
+use futures::stream::TryStreamExt;
+use is_terminal::IsTerminal;
+use wasmer_api::{types::AppTemplate, WasmerClient};
+use wasmer_config::{app::AppConfigV1, package::PackageSource};
+
+use super::{deploy::CmdAppDeploy, util::login_user};
 use crate::{
     commands::AsyncCliCommand,
     opts::{ApiOpts, ItemFormatOpts, WasmerEnv},
     utils::{load_package_manifest, prompts::PackageCheckMode},
 };
-use anyhow::Context;
-use colored::Colorize;
-use dialoguer::{theme::ColorfulTheme, Confirm, Select};
-use is_terminal::IsTerminal;
-use std::{collections::HashMap, env, io::Cursor, path::PathBuf, str::FromStr};
-use wasmer_api::{types::AppTemplate, WasmerClient};
-use wasmer_config::{app::AppConfigV1, package::PackageSource};
-
-use super::{deploy::CmdAppDeploy, util::login_user};
 
 async fn write_app_config(app_config: &AppConfigV1, dir: Option<PathBuf>) -> anyhow::Result<()> {
     let raw_app_config = app_config.clone().to_yaml()?;
@@ -270,6 +279,106 @@ impl CmdAppCreate {
         Ok(false)
     }
 
+    /// Load cached templates from a file.
+    ///
+    /// Returns an error if the cache file is older than the max age.
+    fn load_cached_templates(
+        path: &Path,
+    ) -> Result<(Vec<AppTemplate>, std::time::Duration), anyhow::Error> {
+        let modified = path.metadata()?.modified()?;
+        let age = modified.elapsed()?;
+
+        let data = std::fs::read_to_string(path)?;
+        match serde_json::from_str::<Vec<AppTemplate>>(&data) {
+            Ok(v) => Ok((v, age)),
+            Err(err) => {
+                std::fs::remove_file(path).ok();
+                Err(err).context("could not deserialize cached file")
+            }
+        }
+    }
+
+    fn persist_template_cache(path: &Path, templates: &[AppTemplate]) -> Result<(), anyhow::Error> {
+        if let Some(parent) = path.parent() {
+            std::fs::create_dir_all(parent).context("could not create cache dir")?;
+        }
+
+        let data = serde_json::to_vec(templates)?;
+
+        std::fs::write(path, data)?;
+        tracing::trace!(path=%path.display(), "persisted app template cache");
+
+        Ok(())
+    }
+
+    /// Tries to retrieve templates from a local file cache.
+    /// Fetches the templates from the backend if the file doesn't exist,
+    /// can't be loaded, or is older then the max age,
+    async fn fetch_templates_cached(
+        client: &WasmerClient,
+        cache_dir: &Path,
+    ) -> Result<Vec<AppTemplate>, anyhow::Error> {
+        const MAX_CACHE_AGE: Duration = Duration::from_secs(60 * 60);
+        const MAX_COUNT: usize = 100;
+        const CACHE_FILENAME: &str = "app_templates.json";
+
+        let cache_path = cache_dir.join(CACHE_FILENAME);
+
+        let cached_items = match Self::load_cached_templates(&cache_path) {
+            Ok((items, age)) => {
+                if age <= MAX_CACHE_AGE {
+                    return Ok(items);
+                }
+                items
+            }
+            Err(e) => {
+                tracing::trace!(error = &*e, "could not load templates from local cache");
+                Vec::new()
+            }
+        };
+
+        // Either no cache present, or cache has exceeded max age.
+        // Fetch the first page.
+        // If first item matches, then no need to re-fetch.
+        let mut stream = Box::pin(wasmer_api::query::fetch_all_app_templates(
+            client,
+            10,
+            Some(wasmer_api::types::AppTemplatesSortBy::Newest),
+        ));
+
+        let first_page = match stream.try_next().await? {
+            Some(items) => items,
+            None => return Ok(Vec::new()),
+        };
+
+        if let (Some(a), Some(b)) = (cached_items.first(), first_page.first()) {
+            if a == b {
+                // Cached items are up to date, no need to query more.
+                return Ok(cached_items);
+            }
+        }
+
+        let mut items = first_page;
+        while let Some(next) = stream.try_next().await? {
+            items.extend(next);
+
+            if items.len() >= MAX_COUNT {
+                break;
+            }
+        }
+
+        // Persist to cache.
+        if let Err(err) = Self::persist_template_cache(&cache_path, &items) {
+            tracing::trace!(error = &*err, "could not persist template cache");
+        }
+
+        // TODO: sort items by popularity!
+        // Since we can't rely on backend sorting because of the cache
+        // preservation logic, the backend needs to add a popluarity field.
+
+        Ok(items)
+    }
+
     // A utility function used to fetch the URL of the template to use.
     async fn get_template_url(&self, client: &WasmerClient) -> anyhow::Result<url::Url> {
         let mut url = if let Some(template) = &self.template {
@@ -287,17 +396,10 @@ impl CmdAppCreate {
                 anyhow::bail!("No template selected")
             }
 
-            let templates: Vec<AppTemplate> =
-                wasmer_api::query::fetch_app_templates(client, String::new(), 10)
-                    .await?
-                    .ok_or(anyhow::anyhow!("No template received from the backend"))?
-                    .edges
-                    .into_iter()
-                    .flatten()
-                    .filter_map(|v| v.node)
-                    .collect();
+            let templates = Self::fetch_templates_cached(client, &self.env.cache_dir).await?;
 
             let theme = ColorfulTheme::default();
+
             let items = templates
                 .iter()
                 .map(|t| {
@@ -315,10 +417,12 @@ impl CmdAppCreate {
                 })
                 .collect::<Vec<_>>();
 
+            // Note: this should really use `dialoger::FuzzySelect`, but that
+            // breaks the formatting.
             let dialog = dialoguer::Select::with_theme(&theme)
                 .with_prompt(format!("Select a template ({} available)", items.len()))
                 .items(&items)
-                .max_length(6)
+                .max_length(10)
                 .clear(true)
                 .report(false)
                 .default(0);
