@@ -58,6 +58,13 @@ use crate::{bin_factory::BinaryPackage, state::PreopenedDir, ALL_RIGHTS};
 /// run the libc initialization logic
 pub const VIRTUAL_ROOT_FD: WasiFd = 3;
 
+/// The root inode and stdio inodes are the first inodes in the
+/// file system tree
+pub const FS_STDIN_INO: Inode = Inode(10);
+pub const FS_STDOUT_INO: Inode = Inode(11);
+pub const FS_STDERR_INO: Inode = Inode(12);
+pub const FS_ROOT_INO: Inode = Inode(13);
+
 const STDIN_DEFAULT_RIGHTS: Rights = {
     // This might seem a bit overenineered, but it's the only way I
     // discovered for getting the values in a const environment
@@ -95,6 +102,10 @@ impl Inode {
     pub fn as_u64(&self) -> u64 {
         self.0
     }
+
+    pub fn from_path(str: &str) -> Self {
+        Inode(xxhash_rust::xxh64::xxh64(str.as_bytes(), 0))
+    }
 }
 
 #[derive(Debug, Clone)]
@@ -143,7 +154,6 @@ impl InodeWeakGuard {
 #[derive(Debug)]
 #[cfg_attr(feature = "enable-serde", derive(Serialize, Deserialize))]
 struct WasiInodesProtected {
-    seed: u64,
     lookup: HashMap<Inode, Weak<InodeVal>>,
 }
 
@@ -157,7 +167,6 @@ impl WasiInodes {
     pub fn new() -> Self {
         Self {
             protected: Arc::new(RwLock::new(WasiInodesProtected {
-                seed: 1,
                 lookup: Default::default(),
             })),
         }
@@ -166,20 +175,17 @@ impl WasiInodes {
     /// adds another value to the inodes
     pub fn add_inode_val(&self, val: InodeVal) -> InodeGuard {
         let val = Arc::new(val);
+        let st_ino = {
+            let guard = val.stat.read().unwrap();
+            guard.st_ino
+        };
 
         let mut guard = self.protected.write().unwrap();
-        let ino = Inode(guard.seed);
-        guard.seed += 1;
+        let ino = Inode(st_ino);
         guard.lookup.insert(ino, Arc::downgrade(&val));
 
-        // Set the inode value
-        {
-            let mut guard = val.stat.write().unwrap();
-            guard.st_ino = ino.0;
-        }
-
         // every 100 calls we clear out dead weaks
-        if guard.seed % 100 == 1 {
+        if guard.lookup.len() % 100 == 1 {
             guard.lookup.retain(|_, v| Weak::strong_count(v) > 0);
         }
 
@@ -591,7 +597,7 @@ impl WasiFs {
         vfs_preopens: &[String],
         fs_backing: WasiFsRoot,
     ) -> Result<Self, String> {
-        let mut wasi_fs = Self::new_init(fs_backing, inodes)?;
+        let mut wasi_fs = Self::new_init(fs_backing, inodes, FS_ROOT_INO)?;
         wasi_fs.init_preopens = preopens.to_vec();
         wasi_fs.init_vfs_preopens = vfs_preopens.to_vec();
         wasi_fs.create_preopens(inodes, false)?;
@@ -612,11 +618,16 @@ impl WasiFs {
 
     /// Private helper function to init the filesystem, called in `new` and
     /// `new_with_preopen`
-    fn new_init(fs_backing: WasiFsRoot, inodes: &WasiInodes) -> Result<Self, String> {
+    fn new_init(
+        fs_backing: WasiFsRoot,
+        inodes: &WasiInodes,
+        st_ino: Inode,
+    ) -> Result<Self, String> {
         debug!("Initializing WASI filesystem");
 
         let stat = Filestat {
             st_filetype: Filetype::Directory,
+            st_ino: st_ino.as_u64(),
             ..Filestat::default()
         };
         let root_kind = Kind::Root {
@@ -919,7 +930,7 @@ impl WasiFs {
         &self,
         inodes: &WasiInodes,
         mut cur_inode: InodeGuard,
-        path: &str,
+        path_str: &str,
         mut symlink_count: u32,
         follow_symlinks: bool,
     ) -> Result<InodeGuard, Errno> {
@@ -927,7 +938,7 @@ impl WasiFs {
             return Err(Errno::Mlink);
         }
 
-        let path: &Path = Path::new(path);
+        let path: &Path = Path::new(path_str);
         let n_components = path.components().count();
 
         // TODO: rights checks
@@ -1048,6 +1059,11 @@ impl WasiFs {
                                         file.to_string_lossy().to_string().into(),
                                         Filestat {
                                             st_filetype: file_type,
+                                            st_ino: Inode::from_path(path_str).as_u64(),
+                                            st_size: metadata.len(),
+                                            st_ctim: metadata.created(),
+                                            st_mtim: metadata.modified(),
+                                            st_atim: metadata.accessed(),
                                             ..Filestat::default()
                                         },
                                     );
@@ -1546,14 +1562,15 @@ impl WasiFs {
             _ => {}
         }
 
-        let ret = inodes.add_inode_val(InodeVal {
+        let st_ino = Inode::from_path(&name);
+        stat.st_ino = st_ino.as_u64();
+
+        inodes.add_inode_val(InodeVal {
             stat: RwLock::new(stat),
             is_preopened,
             name,
             kind: RwLock::new(kind),
-        });
-        stat.st_ino = ret.ino().as_u64();
-        ret
+        })
     }
 
     pub fn create_fd(
@@ -1565,15 +1582,19 @@ impl WasiFs {
         inode: InodeGuard,
     ) -> Result<WasiFd, Errno> {
         let idx = self.next_fd.next_val();
-        self.create_fd_ext(rights, rights_inheriting, flags, open_flags, inode, idx)?;
+        self.create_fd_ext(
+            rights,
+            rights_inheriting,
+            flags,
+            open_flags,
+            inode,
+            idx,
+            false,
+        )?;
         Ok(idx)
     }
 
-    pub fn make_max_fd(&self, fd: u32) {
-        self.next_fd.clip_val(fd);
-    }
-
-    pub fn create_fd_ext(
+    pub fn with_fd(
         &self,
         rights: Rights,
         rights_inheriting: Rights,
@@ -1581,12 +1602,44 @@ impl WasiFs {
         open_flags: u16,
         inode: InodeGuard,
         idx: WasiFd,
+    ) -> Result<(), Errno> {
+        self.make_max_fd(idx + 1);
+        self.create_fd_ext(
+            rights,
+            rights_inheriting,
+            flags,
+            open_flags,
+            inode,
+            idx,
+            true,
+        )?;
+        Ok(())
+    }
+
+    pub fn make_max_fd(&self, fd: u32) {
+        self.next_fd.clip_val(fd);
+    }
+
+    #[allow(clippy::too_many_arguments)]
+    pub fn create_fd_ext(
+        &self,
+        rights: Rights,
+        rights_inheriting: Rights,
+        flags: Fdflags,
+        open_flags: u16,
+        inode: InodeGuard,
+        idx: WasiFd,
+        exclusive: bool,
     ) -> Result<(), Errno> {
         let is_stdio = matches!(
             idx,
             __WASI_STDIN_FILENO | __WASI_STDOUT_FILENO | __WASI_STDERR_FILENO
         );
-        self.fd_map.write().unwrap().insert(
+        let mut guard = self.fd_map.write().unwrap();
+        if exclusive && guard.contains_key(&idx) {
+            return Err(Errno::Exist);
+        }
+        guard.insert(
             idx,
             Fd {
                 rights,
@@ -1640,6 +1693,7 @@ impl WasiFs {
             __WASI_STDOUT_FILENO,
             STDOUT_DEFAULT_RIGHTS,
             Fdflags::APPEND,
+            FS_STDOUT_INO,
         );
     }
 
@@ -1651,6 +1705,7 @@ impl WasiFs {
             __WASI_STDIN_FILENO,
             STDIN_DEFAULT_RIGHTS,
             Fdflags::empty(),
+            FS_STDIN_INO,
         );
     }
 
@@ -1662,6 +1717,7 @@ impl WasiFs {
             __WASI_STDERR_FILENO,
             STDERR_DEFAULT_RIGHTS,
             Fdflags::APPEND,
+            FS_STDERR_INO,
         );
     }
 
@@ -1873,6 +1929,7 @@ impl WasiFs {
         Ok(())
     }
 
+    #[allow(clippy::too_many_arguments)]
     pub(crate) fn create_std_dev_inner(
         &self,
         inodes: &WasiInodes,
@@ -1881,10 +1938,12 @@ impl WasiFs {
         raw_fd: WasiFd,
         rights: Rights,
         fd_flags: Fdflags,
+        st_ino: Inode,
     ) {
         let inode = {
             let stat = Filestat {
                 st_filetype: Filetype::CharacterDevice,
+                st_ino: st_ino.as_u64(),
                 ..Filestat::default()
             };
             let kind = Kind::File {
@@ -1921,6 +1980,7 @@ impl WasiFs {
                     let wf = wf.read().unwrap();
                     return Ok(Filestat {
                         st_filetype: Filetype::RegularFile,
+                        st_ino: Inode::from_path(path.to_string_lossy().as_ref()).as_u64(),
                         st_size: wf.size(),
                         st_atim: wf.last_accessed(),
                         st_mtim: wf.last_modified(),
