@@ -70,9 +70,9 @@ impl JournalEffector {
             .create_inode_with_default_stat(inodes, kind, false, "socket".into());
 
         let rights = Rights::all_socket();
-        let ret_fd = state
+        state
             .fs
-            .create_fd(rights, rights, Fdflags::empty(), 0, inode)
+            .with_fd(rights, rights, Fdflags::empty(), 0, inode, fd)
             .map_err(|err| {
                 anyhow::format_err!(
                     "journal restore error: failed to create remote connected socket - {}",
@@ -80,16 +80,6 @@ impl JournalEffector {
                 )
             })?;
 
-        let ret = crate::syscalls::fd_renumber_internal(ctx, ret_fd, fd);
-        if ret != Errno::Success {
-            bail!(
-                    "journal restore error: failed renumber file descriptor after connecting the socket (from={}, to={}) - {}",
-                    ret_fd,
-                    fd,
-                    ret
-                );
-        }
-
         Ok(())
     }
 }
