@@ -18,6 +18,8 @@ use crate::{
     WasiEnvBuilder,
 };
 
+pub const MAPPED_CURRENT_DIR_DEFAULT_PATH: &str = "/home";
+
 #[derive(Debug, Clone)]
 pub struct MappedCommand {
     /// The new alias.
@@ -34,6 +36,8 @@ pub(crate) struct CommonWasiOptions {
     pub(crate) forward_host_env: bool,
     pub(crate) mapped_host_commands: Vec<MappedCommand>,
     pub(crate) mounts: Vec<MountedDirectory>,
+    pub(crate) is_home_mapped: bool,
+    pub(crate) is_tmp_mapped: bool,
     pub(crate) injected_packages: Vec<BinaryPackage>,
     pub(crate) capabilities: Capabilities,
     #[derivative(Debug = "ignore")]
@@ -52,7 +56,11 @@ impl CommonWasiOptions {
         wasi: &WasiAnnotation,
         root_fs: Option<TmpFileSystem>,
     ) -> Result<(), anyhow::Error> {
-        let root_fs = root_fs.unwrap_or_else(|| RootFileSystemBuilder::default().build());
+        let root_fs = root_fs.unwrap_or_else(|| {
+            RootFileSystemBuilder::default()
+                .with_tmp(!self.is_tmp_mapped)
+                .build()
+        });
         let fs = prepare_filesystem(root_fs, &self.mounts, container_fs)?;
 
         builder.add_preopen_dir("/")?;
