@@ -27,7 +27,7 @@ pub fn sock_recv<M: MemorySize>(
     ro_flags: WasmPtr<RoFlags, M>,
 ) -> Result<Errno, WasiError> {
     let env = ctx.data();
-    let fd_entry = env.state.fs.get_fd(sock).unwrap();
+    let fd_entry = wasi_try_ok!(env.state.fs.get_fd(sock));
     let guard = fd_entry.inode.read();
     let use_read = matches!(guard.deref(), Kind::Pipe { .. });
     drop(guard);
